package com.hichina.main.back.hichinamainback.auth;

import com.google.api.client.googleapis.auth.oauth2.GoogleIdToken;
import com.google.api.client.googleapis.auth.oauth2.GoogleIdTokenVerifier;
import com.google.api.client.http.javanet.NetHttpTransport;
import com.google.api.client.json.gson.GsonFactory;
import com.hichina.main.back.hichinamainback.utils.ProxyUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.InternalAuthenticationServiceException;
import org.springframework.security.core.AuthenticationException;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.security.GeneralSecurityException;
import java.util.Collections;

@Component
public class GoogleAuthenticator {

    private static final Logger LOGGER = LoggerFactory.getLogger(GoogleAuthenticator.class);

    static {
        System.setProperty("https.protocols", "TLSv1.2");
    }

    @Value("${google.authenticator.clientId}")
    private String clientId;

    public AuthUserInfo verify(String token) throws AuthenticationException {

        GoogleIdTokenVerifier verifier = new GoogleIdTokenVerifier.Builder(
                new NetHttpTransport.Builder().setProxy(ProxyUtils.defaultProxy()).build(),
                new GsonFactory())
                // Specify the CLIENT_ID of the app that accesses the backend:
                .setAudience(Collections.singletonList(clientId))
                .build();

        try {
            GoogleIdToken idToken = verifier.verify(token);
            if (idToken != null) {
                GoogleIdToken.Payload payload = idToken.getPayload();

                String userId = payload.getSubject();

                // Get profile information from payload
                String email = payload.getEmail();
                String name = (String) payload.get("name");
                String pictureUrl = (String) payload.get("picture");

                return new AuthUserInfo(userId, name, email, pictureUrl);
            } else {
                LOGGER.error("Invalid Google ID token: {}", token);
                throw new BadCredentialsException("Invalid ID token.");
            }
        } catch (GeneralSecurityException | IOException e) {
            LOGGER.error("Google auth internal Error: {}", e.getMessage(), e);
            throw new InternalAuthenticationServiceException("Internal Error", e);
        }
    }

}
